@@ -24,7 +24,9 @@ package net.sourceforge.jnlp;
 
 import java.net.URL;
 import java.net.MalformedURLException;
+import java.util.HashSet;
 import java.util.Hashtable;
+import java.util.Iterator;
 import java.util.Locale;
 import java.util.List;
 import java.util.ArrayList;
@@ -35,38 +37,52 @@ import net.sourceforge.jnlp.runtime.JNLPRuntime;
 public class PluginBridge extends JNLPFile {
 
     String name;
-    String[] jars = new String[0];
+    HashSet<String> jars = new HashSet<String>();
     String[] cacheJars = new String[0];
     String[] cacheExJars = new String[0];
     Hashtable<String, String> atts;
     private boolean usePack;
     private boolean useVersion;
     private boolean codeBaseLookup;
+    private boolean useJNLPHref;
 
     public PluginBridge(URL codebase, URL documentBase, String jar, String main,
-                        int width, int height, Hashtable<String, String> atts)
+                        int width, int height, Hashtable<String, String> atts,
+                        String uKey)
             throws Exception {
         specVersion = new Version("1.0");
         fileVersion = new Version("1.1");
         this.codeBase = codebase;
         this.sourceLocation = documentBase;
+        this.atts = atts;
 
         if (atts.containsKey("jnlp_href")) {
+            useJNLPHref = true;
             try {
                 URL jnlp = new URL(codeBase.toExternalForm() + atts.get("jnlp_href"));
-                JNLPFile jnlpFile = new JNLPFile(jnlp);
+                JNLPFile jnlpFile = new JNLPFile(jnlp, null, false, JNLPRuntime.getDefaultUpdatePolicy(), this.codeBase);
                 Map<String, String> jnlpParams = jnlpFile.getApplet().getParameters();
+                info = jnlpFile.info;
 
                 // Change the parameter name to lowercase to follow conventions.
                 for (Map.Entry<String, String> entry : jnlpParams.entrySet()) {
-                    atts.put(entry.getKey().toLowerCase(), entry.getValue());
+                    this.atts.put(entry.getKey().toLowerCase(), entry.getValue());
                 }
+                JARDesc[] jarDescs = jnlpFile.getResources().getJARs();
+                for (JARDesc jarDesc : jarDescs) {
+                     String fileName = jarDesc.getLocation().toExternalForm();
+                     this.jars.add(fileName);
+                 }
             } catch (MalformedURLException e) {
                 // Don't fail because we cannot get the jnlp file. Parameters are optional not required.
                 // it is the site developer who should ensure that file exist.
                 System.err.println("Unable to get JNLP file at: " + codeBase.toExternalForm()
                         + atts.get("jnlp_href"));
             }
+        } else {
+            // Should we populate this list with applet attribute tags?
+            info = new ArrayList<InformationDesc>();
+            useJNLPHref = false;
         }
 
         // also, see if cache_archive is specified
@@ -100,11 +116,13 @@ public class PluginBridge extends JNLPFile {
         }
 
         if (jar != null && jar.length() > 0) {
-            this.jars = jar.split(",");
+            String[] jars = jar.split(",");
 
             // trim white spaces
-            for (int i = 0; i < this.jars.length; i++) {
-                this.jars[i] = this.jars[i].trim();
+            for (int i = 0; i < jars.length; i++) {
+                String jarName = jars[i].trim();
+                if (jarName.length() > 0)
+                    this.jars.add(jarName);
             }
 
             if (JNLPRuntime.isDebug()) {
@@ -112,7 +130,6 @@ public class PluginBridge extends JNLPFile {
                 System.err.println("jars length: " + jars.length);
             }
         }
-        this.atts = atts;
 
         name = atts.get("name");
         if (name == null)
@@ -123,7 +140,9 @@ public class PluginBridge extends JNLPFile {
         if (main.endsWith(".class"))
             main = main.substring(0, main.length() - 6);
 
-        launchType = new AppletDesc(name, main, documentBase, width,
+        // the class name should be of the form foo.bar.Baz not foo/bar/Baz
+        String mainClass = main.replace('/', '.');
+        launchType = new AppletDesc(name, mainClass, documentBase, width,
                                     height, atts);
 
         if (main.endsWith(".class")) //single class file only
@@ -132,34 +151,7 @@ public class PluginBridge extends JNLPFile {
         else
             security = null;
 
-        /* According to http://download.oracle.com/javase/6/docs/technotes/guides/deployment/deployment-guide/applet-compatibility.html, 
-         * classloaders are shared iff these properties match:
-         * codebase, cache_archive, java_archive, archive
-         * 
-         * To achieve this, we create the uniquekey based on those 4 values,
-         * always in the same order. The initial "<NAME>=" parts ensure a 
-         * bad tag cannot trick the loader into getting shared with another.
-         */
-
-        // Firefox sometimes skips the codebase if it is default  -- ".", 
-        // so set it that way if absent
-        String codebaseAttr =      atts.get("codebase") != null ?
-                                   atts.get("codebase") : ".";
-
-        String cache_archiveAttr = atts.get("cache_archive") != null ? 
-                                   atts.get("cache_archive") : "";
-
-        String java_archiveAttr =  atts.get("java_archive") != null ? 
-                                   atts.get("java_archive") : "";
-
-        String archiveAttr =       atts.get("archive") != null ? 
-                                   atts.get("archive") : "";
-
-        this.uniqueKey = "codebase=" + codebaseAttr +
-                         "cache_archive=" + cache_archiveAttr + 
-                         "java_archive=" + java_archiveAttr + 
-                         "archive=" +  archiveAttr;
-
+        this.uniqueKey = uKey;
         usePack = false;
         useVersion = false;
         String jargs = atts.get("java_arguments");
@@ -183,6 +175,10 @@ public class PluginBridge extends JNLPFile {
     	return codeBaseLookup;
     }
 
+    public boolean useJNLPHref() {
+        return useJNLPHref;
+    }
+
     /**
      * {@inheritdoc }
      */
@@ -195,15 +191,6 @@ public class PluginBridge extends JNLPFile {
         return name;
     }
 
-    public InformationDesc getInformation(final Locale locale) {
-        return new InformationDesc(this, new Locale[] { locale }) {
-            protected List<Object> getItems(Object key) {
-                // Should we populate this list with applet attribute tags?
-                return new ArrayList<Object>();
-            }
-        };
-    }
-
     public ResourcesDesc getResources(final Locale locale, final String os,
                                       final String arch) {
         return new ResourcesDesc(this, new Locale[] { locale }, new String[] { os },
@@ -217,10 +204,13 @@ public class PluginBridge extends JNLPFile {
                         List<JARDesc> jarDescs = new ArrayList<JARDesc>();
                         jarDescs.addAll(sharedResources.getResources(JARDesc.class));
 
-                        for (int i = 0; i < jars.length; i++)
-                            if (jars[i].length() > 0)
-                                jarDescs.add(new JARDesc(new URL(codeBase, jars[i]),
+                        Iterator<String> it = jars.iterator();
+                        while(it.hasNext()) {
+                            String name = it.next();
+                            if (name.length() > 0)
+                                jarDescs.add(new JARDesc(new URL(codeBase, name),
                                         null, null, false, true, false, true));
+                        }
 
                         boolean cacheable = true;
 
