@@ -16,8 +16,6 @@
 
 package net.sourceforge.jnlp;
 
-import java.io.*;
-import java.net.*;
 import java.util.*;
 
 /**
