@@ -194,7 +194,7 @@ public class JarSigner implements CertVerifier {
                 // some sort of resource download/cache error. Nothing to add
                 // in that case ... but don't fail here
                 if (jarFile == null) {
-                    return;
+                    continue;
                 }
 
                 String localFile = jarFile.getAbsolutePath();
@@ -373,7 +373,13 @@ public class JarSigner implements CertVerifier {
                 alreadyTrustPublisher = CertificateUtils.inKeyStores(publisher, certKeyStores);
                 X509Certificate root = (X509Certificate) getRoot();
                 KeyStore[] caKeyStores = KeyStores.getCAKeyStores();
-                rootInCacerts = CertificateUtils.inKeyStores(root, caKeyStores);
+                // Check entire cert path for a trusted CA
+                for (Certificate c : certPath.getCertificates()) {
+                	if ((rootInCacerts = CertificateUtils.inKeyStores(
+                            (X509Certificate) c, caKeyStores))) {
+                        break;
+                    }
+                }
             } catch (Exception e) {
                 // TODO: Warn user about not being able to
                 // look through their cacerts/trusted.certs
